@@ -1,4 +1,4 @@
-relate.levels<-function(lower, upper, defuzzify=FALSE, verbose=FALSE, ...) {
+relate.levels<-function(lower, upper, defuzzify=FALSE, excludeFixed = FALSE, verbose=FALSE, ...) {
   minupperclasses =999999
   maxupperclasses =0
 	minlowerclasses=999999
@@ -30,17 +30,27 @@ relate.levels<-function(lower, upper, defuzzify=FALSE, verbose=FALSE, ...) {
 	for(j in 1:length(lower)) {
 		if(!is.null(lower[[j]])) {
 			nlowerclasses = length(names(lower[[j]]$dist2clusters))
+			colind<-nlowerclasses-minlowerclasses+1
+			if(excludeFixed) {
+			  nfixed = sum(substr(names(lower[[j]]$dist2clusters),1,1)=="F")
+        nlowerclasses = nlowerclasses - nfixed
+        colind<-nlowerclasses-minlowerclasses+1 + nfixed
+			}
 			if(verbose) cat(paste("Number of lower classes:", nlowerclasses,"\n"))
 			#Loop over upper clustering
 			for(i in 1:length(upper)) {
 				if(!is.null(upper[[i]])){
 					if(verbose) cat(".")
 					nupperclasses = length(names(upper[[i]]$dist2clusters))
+					rowind<-nupperclasses-minupperclasses+1
+					if(excludeFixed) {
+            nfixed = sum(substr(names(upper[[i]]$dist2clusters),1,1)=="F")
+            nupperclasses = nupperclasses - nfixed
+            rowind<-nupperclasses-minupperclasses+1 + nfixed
+					}
 					memb<-crossmemb(lower[[j]], upper[[i]])
 					if(lower[[j]]$method=="NC") memb = memb[-nrow(memb),]
 					if(defuzzify) memb <- defuzzify(memb, ...)$memb
-					rowind<-nupperclasses-minupperclasses+1
-					colind<-nlowerclasses-minlowerclasses+1
 					minmaxall[rowind,colind] = min(apply(memb[1:nlowerclasses,1:nupperclasses],2,max))
 					minallsize[rowind,colind] = min(apply(memb[1:nlowerclasses,1:nupperclasses],2,sum))
 					empty[rowind,colind] = sum(colSums(defuzzify(memb[1:nlowerclasses,1:nupperclasses], method="max")$memb)==0)
@@ -48,7 +58,7 @@ relate.levels<-function(lower, upper, defuzzify=FALSE, verbose=FALSE, ...) {
 					if(upper[[i]]$method=="NC") {
 						maxnoise[rowind,colind]=max(memb[1:nlowerclasses,ncol(memb)])
 						nplots = colSums(defuzzify(lower[[j]], method="cut", alpha=0.5)$memb)
-						nnoise[rowind,colind]=sum(memb[1:nlowerclasses,ncol(memb)]>0.5 & nplots[1:nlowerclasses]>=20)
+						nnoise[rowind,colind]=sum(memb[1:nlowerclasses,ncol(memb)]>0.5)
 					 	#print(nnoise[rowind,colind])
 					}
 				}
